@@ -13,7 +13,7 @@ use 5.010001;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.822';
+our $VERSION = '1.823';
 
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
 
@@ -68,8 +68,11 @@ sub DESTROY {
 
    if ($_Q->{_init_pid} eq $_pid) {
       MCE::Util::_destroy_socks($_Q, qw(_aw_sock _ar_sock _qw_sock _qr_sock));
-      for my $_i (0 .. MUTEX_LOCKS - 1) {
-         delete $_Q->{'_mutex_'.$_i};
+
+      if (exists $_Q->{_mutex_0}) {
+         for my $_i (0 .. MUTEX_LOCKS - 1) {
+            delete $_Q->{'_mutex_'.$_i};
+         }
       }
    }
 
@@ -132,8 +135,12 @@ sub new {
    $_Q->{_init_pid} = $_has_threads ? $$ .'.'. $_tid : $$;
    $_Q->{_dsem} = 0 if ($_Q->{_fast});
 
-   for my $_i (0 .. MUTEX_LOCKS - 1) {
-      $_Q->{'_mutex_'.$_i} = MCE::Mutex->new( impl => 'Channel' );
+   my $_caller = caller() eq 'MCE::Shared' ? caller(1) : caller();
+
+   if ($_caller !~ /^MCE::/ && $_tid == 0 && $_Q->{_fast} == 0) {
+      for my $_i (0 .. MUTEX_LOCKS - 1) {
+         $_Q->{'_mutex_'.$_i} = MCE::Mutex->new( impl => 'Channel' );
+      }
    }
 
    MCE::Util::_sock_pair($_Q, qw(_qr_sock _qw_sock));
@@ -683,7 +690,7 @@ MCE::Shared::Queue - Hybrid-queue helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Queue version 1.822
+This document describes MCE::Shared::Queue version 1.823
 
 =head1 DESCRIPTION
 
